
# q-expansion of newform 27.5.b.c, downloaded from the LMFDB on 12 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 27
weight = 5
poly_data = [1, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 3], 1]]

hecke_ring_character_values = [[2, [-1, 0]]]
aps_data = [[0, 1], [0, 0], [0, 11], [-19, 0], [0, -41], [302, 0], [0, -138], [-304, 0], [0, -100], [0, 226], [239, 0], [740, 0], [0, -76], [-982, 0], [0, -722], [0, 531], [0, 974], [-316, 0], [4622, 0], [0, -606], [-3031, 0], [-10450, 0], [0, -4211], [0, 2334], [-6517, 0], [0, -1973], [-7654, 0], [0, 171], [2324, 0], [0, 1640], [24995, 0], [0, -9487], [0, -818], [-11884, 0], [0, 7331], [-2683, 0], [-32116, 0], [22790, 0], [0, 12026], [0, -6575], [0, 16305], [-49552, 0], [0, 15130], [35447, 0], [0, -11913], [-31255, 0], [-15052, 0], [50174, 0], [0, -6422], [34214, 0], [0, 12462], [0, 20600], [41390, 0], [0, 27606], [0, 6530], [0, -5564], [0, -40302], [73739, 0], [11996, 0], [0, -17042], [-1048, 0], [0, -21394], [154154, 0], [0, -31360], [-25903, 0], [0, 32281], [-164854, 0], [148694, 0], [0, 35891], [127520, 0], [0, 47368], [0, 6474], [-151345, 0], [237506, 0], [-261952, 0], [0, 29054], [0, -79781], [217154, 0], [0, 85400], [-199291, 0], [0, -83758], [-30412, 0], [0, -53910], [-213541, 0], [66725, 0], [0, 91390], [0, 77928], [-90667, 0], [0, -67319], [-323977, 0], [0, -25647], [0, -64406], [-34882, 0], [0, -72349], [464810, 0], [0, 55860], [0, -11899], [0, 14250], [-176434, 0], [-323836, 0], [-223390, 0], [0, 195009], [0, 28025], [0, 212464], [80726, 0], [261182, 0], [0, 130435], [0, -100890], [0, 97166], [402173, 0], [-378670, 0], [287570, 0], [0, -192088], [223262, 0], [43373, 0], [0, -141140], [-546088, 0], [0, 139644], [0, -234403], [0, -34007], [230720, 0], [-469369, 0], [0, 114382], [0, -8214], [-266500, 0], [0, -230103], [-105184, 0], [0, 234996], [126089, 0], [97736, 0], [-857158, 0], [0, -303322], [61223, 0], [782570, 0], [0, -233800], [-85045, 0], [0, -214374], [1068548, 0], [0, 179645], [0, 153198], [-961360, 0], [0, -35222], [-493555, 0], [0, -64290], [577226, 0], [0, 23662], [81206, 0], [0, -360146], [503348, 0], [0, 182700], [700034, 0], [0, -268878], [342704, 0], [0, -5374], [1569500, 0], [0, 166966], [1037153, 0], [0, -42692], [-879451, 0], [0, -239419], [0, -24335], [0, -103056], [-366187, 0], [0, 478035], [0, 103934], [0, -230054], [981875, 0], [241946, 0]]
